angular.module('mp3', ['ionic', 'utils', 'mp3.controllers', 'mp3.services', 'mp3.stores'])

    //lodash
    .constant('_', window._)

    //localStorage keys
    .constant('storageKeys', {
        IS_INITIALIZED: 'isInitialized',
        PARTS_PROGRESS: 'parts',
        ACHIEVEMENTS_PROGRESS: 'achievements'
    })

    //.filter('unsafe', function($sce) { return $sce.trustAsHtml; })

    .run(function ($ionicPlatform, ChaptersService) {
        $ionicPlatform.ready(function () {
            // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
            // for form inputs)
            if (window.cordova && window.cordova.plugins.Keyboard) {
                cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
            }
            if (window.StatusBar) {
                // org.apache.cordova.statusbar required
                StatusBar.styleDefault();
            }
            //
            ChaptersService.init();

            if (( /(ipad|iphone|ipod|android|windows phone)/i.test(navigator.userAgent) )) {
                document.addEventListener('deviceready', initApp, false);
            } else {
                initApp();
            }
            //
            function initApp() {
                if (! window.AdMob ) {
                    console.log('Admob Plugin Unavailable');
                } else {
                    //Admob
                    var admobid = {
                        banner: 'ca-app-pub-4265932978799476/1509542943',
                        interstitial: 'ca-app-pub-6869992474017983/1657046752'
                    };
                    AdMob.createBanner({
                        adId: admobid.banner,
                        isTesting: false,
                        overlap: false,
                        offsetTopBar: false,
                        position: AdMob.AD_POSITION.BOTTOM_CENTER,
                        bgColor: 'black'
                    });
                }
                //Google analytics
                if (! window.analytics ) {
                    console.log("Google Analytics Plugin Unavailable");
                } else {
                    window.analytics.debugMode();
                    window.analytics.startTrackerWithId('UA-66432610-1');
                    window.analytics.trackEvent('General', 'App Init');
                }
            }

        });
    })

    .config(function ($stateProvider, $urlRouterProvider) {
        $stateProvider

            .state('app', {
                url: "/app",
                abstract: true,
                templateUrl: "templates/menu.html",
                controller: 'AppCtrl'
            })

            .state('app.home', {
                url: "/home",
                views: {
                    'menuContent': {
                        templateUrl: "templates/home.html",
                        controller: 'HomeCtrl'
                    }
                }
            })

            .state('app.about', {
                url: "/about",
                views: {
                    'menuContent': {
                        templateUrl: "templates/about.html",
                        controller: 'AboutCtrl'
                    }
                }
            })

            .state('app.chapters', {
                url: "/chapters",
                views: {
                    'menuContent': {
                        templateUrl: "templates/chapters.html",
                        controller: 'ChaptersCtrl'
                    }
                }
            })

            .state('app.chapter', {
                url: "/chapters/:chapterId",
                views: {
                    'menuContent': {
                        templateUrl: "templates/chapter.html",
                        controller: 'ChapterCtrl'
                    }
                }
            })

            .state('app.achievements', {
                url: "/achievements",
                views: {
                    'menuContent': {
                        templateUrl: "templates/achievements.html",
                        controller: 'AchievementsCtrl'
                    }
                }
            })
        ;

        // if none of the above states are matched, use this as the fallback
        $urlRouterProvider.otherwise('/app/home');
    }
);

angular.module('utils', [])
    .factory('$localStorage', function($window) {
        return {
            set: function(key, value) {
                $window.localStorage[key] = value;
            },
            get: function(key, defaultValue) {
                return $window.localStorage[key] || defaultValue;
            },
            setObject: function(key, value) {
                $window.localStorage[key] = JSON.stringify(value);
            },
            getObject: function(key) {
                return JSON.parse($window.localStorage[key] || '{}');
            }
        }
    })
    .factory('$utils', function (_) {
        return {
            upsert: function (arr, key, newval) {
                var match = _.find(arr, key);
                if(match){
                    var index = _.indexOf(arr, _.find(arr, key));
                    arr.splice(index, 1, newval);
                } else {
                    arr.push(newval);
                }
            }
        }
});