angular.module('mp3.controllers', [])

    .controller('HomeCtrl', function (GoogleAnalytics) {
        GoogleAnalytics.trackView('Home');
    })

    .controller('AboutCtrl', function (GoogleAnalytics) {
        GoogleAnalytics.trackView('About');
    })

    .controller('AppCtrl', function ($scope, GoogleAnalytics) {
        $scope.quit = function () {
            ionic.Platform.exitApp();
            GoogleAnalytics.trackEvent('Menu', 'Exit App');
        }
    })

    .controller('AchievementsCtrl', function ($scope, AchievementsService, GoogleAnalytics, $ionicLoading, _) {
        var getAllAchievements = function () {
            $ionicLoading.show();
            AchievementsService.getAchievements()
                .then(function (achievements) {
                    $scope.achievements = achievements;
                    $scope.achievementsBase = achievements;
                    $ionicLoading.hide();
                    GoogleAnalytics.trackView('Achievements');
                });
        };

        getAllAchievements();

        /**
         * Update completed change
         * @param part
         */
        $scope.pushCompletedChange = function (achievement) {
            console.log(arguments);
            //Update in localStorage
            var achievementChange = {
                id: achievement.id,
                completed: achievement.completed
            };
            console.log('Saving achievement progress change', achievementChange);
            AchievementsService.saveProgress(achievementChange);
            GoogleAnalytics.trackEvent('Achievements', 'Toggle Completed', achievement.name, achievement.completed);
        };

        /**
         *
         */
        $scope.filterDone = true;
        $scope.toggleCompleted = function () {
            $scope.showAllButton = true;
            if ($scope.filterDone) {
                $scope.achievements = _.filter($scope.achievementsBase, {completed: true});
                $scope.filterDone = false;
            } else {
                $scope.achievements = _.filter($scope.achievementsBase, {completed: false});
                $scope.filterDone = true
            }
            GoogleAnalytics.trackEvent('Achievements', 'Toggle Completed');
        };

        /**
         *
         */
        $scope.showAll = function () {
            $scope.showAllButton = false;
            $scope.achievements = $scope.achievementsBase;
            GoogleAnalytics.trackEvent('Achievements', 'Show All');
        }

        /**
         *
         */
        $scope.filter = function (keywords) {
            keywords = keywords.toLowerCase();
            if (keywords) {
                $scope.achievements = _.filter($scope.achievementsBase, function (achievement) {
                    return achievement.name.toLowerCase().indexOf(keywords)>-1 || achievement.how_to.toLowerCase().indexOf(keywords)>-1 ;
                })
            } else {
                $scope.achievements = $scope.achievementsBase;
            }
        }
    })

    .controller('ChaptersCtrl', function ($rootScope, ChaptersService, GoogleAnalytics, $ionicLoading) {
        ChaptersService.getChapters()
            .then(function (chapters) {
                $rootScope.chapters = chapters;
                $ionicLoading.hide();
                GoogleAnalytics.trackView('Walkthrough');
            });
    })

    .controller('ChapterCtrl', function ($rootScope, $scope, $stateParams, ChaptersService, _, $ionicModal, $compile, $ionicLoading, $ionicScrollDelegate, $ionicPosition, $location, GoogleAnalytics) {
        $scope.parts = null;
        var chapterId = parseInt($stateParams.chapterId);
        $scope.chapter = _.find($rootScope.chapters, {id: chapterId});
        /**
         * Loading Chapter Parts
         */
        ChaptersService.getChapterParts(chapterId)
            .then(function (parts) {
                $scope.parts = parts;
                //
                $ionicLoading.show({
                    template: 'Loading...'
                });
                setTimeout(function () {
                    _.map(parts, function (part) {
                        var html = part.content;
                        // Step 1: parse HTML into DOM element
                        var template = angular.element(html);
                        // Step 2: compile the template
                        var linkFn = $compile(template);
                        // Step 3: link the compiled template with the scope.
                        var element = linkFn($scope);
                        // Step 4: Append to DOM (optional)
                        var container = angular.element(document.querySelector('#part_' + part.id))
                        container.append(element);
                    });
                    $ionicLoading.hide();

                    //
                    GoogleAnalytics.trackView('Walkthrough/Chapter-' + chapterId + '-' + $scope.chapter.title);
                }, 100);
            });
        /**
         * Modal's
         */
        $ionicModal.fromTemplateUrl('image-modal.html', {
            scope: $scope,
            animation: 'slide-in-up'
        }).then(function (modal) {
            $scope.modal = modal;
        });
        $scope.openModal = function () {
            $scope.modal.show();
            GoogleAnalytics.trackEvent('Walkthrough', 'Image Enlarger Modal', chapterId + '-' + $scope.chapter.title);
        };
        $scope.closeModal = function () {
            $scope.modal.hide();
            $scope.imageSrc = 'img/blank.gif';
        };
        $scope.$on('$destroy', function () {
            $scope.modal.remove();
        });
        /**
         * Update completed count
         * @param part
         */
        $scope.pushCompletedChange = function (part) {
            //Update in localStorage
            var partChange = {
                id: part.id,
                completed: part.completed
            };
            console.log('Saving part progress change', partChange);
            ChaptersService.saveProgress(partChange);

            //Update chapters in $rootScope
            ChaptersService.getChapters()
                .then(function (chapters) {
                    $rootScope.chapters = chapters;
                });

            GoogleAnalytics.trackEvent('Walkthrough', 'Part Completed', chapterId + '-' + $scope.chapter.title + '/' + part.id, part.completed);

        };
        //
        $scope.scrollTo = function(id) {
            $location.hash('completed_'+id);
            $ionicScrollDelegate.anchorScroll(true);
            GoogleAnalytics.trackEvent('Walkthrough', 'scrollTo');
        };

        $scope.scrollTop = function () {
            $ionicScrollDelegate.scrollTop(true);
            GoogleAnalytics.trackEvent('Walkthrough', 'scrollTop');
        }
    })

    .directive('enlarge', function ($ionicLoading) {
        return {
            restrict: 'A',
            link: function (scope, element, attrs) {
                element.on('click', function () {
                    $ionicLoading.show({
                        template: 'Loading... (Internet is required)'
                    });
                    var baseUrl = 'http://www.agafix.org/wp-content/uploads/fay/mp/'
                    scope.imageSrc = baseUrl + attrs.enlarge + '.jpg';
                    scope.openModal();
                });
            }
        };
    })

    .directive('imageloading', function ($ionicLoading) {
        return {
            restrict: 'A',
            link: function (scope, element, attrs) {
                element.bind('load', function () {
                    $ionicLoading.hide();
                });
            }
        };
    })

    .directive('fallbackSrc', function () {
        return {
            restrict: 'A',
            link: function (scope, element, attrs) {
                element.bind('error', function () {
                    angular.element(this).attr("src", attrs.fallbackSrc);
                });
            }
        };
    })
;
