angular.module('mp3.services', [])

    .factory('GoogleAnalytics', function ($window) {
        if (!$window.analytics) {
            console.warn('Analytics unvailable, mocking for browser?');
            $window.analytics = {
                trackView: function () {
                    console.log('Tracking View', arguments);
                },
                trackEvent: function () {
                    console.log('Tracking Event', arguments);
                }
            };
        }
        return $window.analytics;
    })

    .factory('ChaptersService', function (ChapterStore, $localStorage, storageKeys, $q, _, $utils) {

        var getProgressClass = function (chapter) {
            var progress;
            try {
                progress = chapter.completedCount / chapter.totalCount;
            } catch (e) {
                console.error(e);
                progress = 0;
            }
            var progressClass;
            if (progress > 0 && progress < 1) {
                progressClass = 'badge-positive';
            } else if (progress === 1) {
                progressClass = 'badge-balanced';
            } else {
                progressClass = 'badge-assertive'
            }
            return progressClass;
        };

        return {
            /**
             * Init localStorage
             */
            init: function () {
                if ($localStorage.get(storageKeys.IS_INITIALIZED)) {
                    console.log('Yes it is Initialized');
                } else {
                    console.log('No, so Initializing...');
                    var partsProgress = ChapterStore.getPartsProgress();
                    $localStorage.setObject(storageKeys.PARTS_PROGRESS, partsProgress);
                    $localStorage.setObject(storageKeys.ACHIEVEMENTS_PROGRESS, []);
                    $localStorage.setObject(storageKeys.IS_INITIALIZED, true);
                }
            },
            /**
             *
             * @returns {Promise}
             */
            getChapters: function () {
                var dfd = $q.defer();
                var chapters = ChapterStore.getAllChapters();
                chapters = _.map(chapters, function (chapter) {
                    var parts = ChapterStore.getPartsByChapter(chapter.id);
                    chapter.totalCount = _.size(parts);
                    chapter.completedCount = _.size(
                        _.filter(parts, {completed: true})
                    );
                    //Progress Style
                    chapter.progressClass = getProgressClass(chapter);
                    return chapter;
                });
                dfd.resolve(chapters);
                return dfd.promise;
            },
            /**
             *
             * @param chapterId
             * @returns {Promise}
             */
            getChapterParts: function (chapterId) {
                var dfd = $q.defer();
                var parts = ChapterStore.getPartsByChapter(chapterId);
                dfd.resolve(parts);
                return dfd.promise;
            },
            /**
             *
             * @param {Object} {id, completed}
             */
            saveProgress: function (partChange) {
                var storagePartsProgress = $localStorage.getObject(storageKeys.PARTS_PROGRESS);
                $utils.upsert(storagePartsProgress, {id: partChange.id}, partChange);
                $localStorage.setObject(storageKeys.PARTS_PROGRESS, storagePartsProgress);
            }
        };
    })

    .factory('AchievementsService', function (AchievementsStore, $localStorage, storageKeys, $q, _, $utils) {
        return {
            getAchievements: function () {
                var dfd = $q.defer();
                var achievements = AchievementsStore.getAllAchievements();
                dfd.resolve(achievements);
                return dfd.promise;
            },
            /**
             *
             * @param {Object} {id, completed}
             */
            saveProgress: function (achievementChange) {
                var storageAchievementsProgress = $localStorage.getObject(storageKeys.ACHIEVEMENTS_PROGRESS);
                $utils.upsert(storageAchievementsProgress, {id: achievementChange.id}, achievementChange);
                $localStorage.setObject(storageKeys.ACHIEVEMENTS_PROGRESS, storageAchievementsProgress);
            }
        };
    });
